"""
Copyright 2011,  Leif Theden & Michael Bachmann


This file is part of Nein.

Nein is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Nein is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Nein.  If not, see <http://www.gnu.org/licenses/>.
"""

from sprite import PhysicsSprite, SpriteBundle

class Actor(object):
    """
    An actor is a sprite or bundle has has life, can be injured, etc
    """

    def __init__(self, hp=100, max_hp=100):
        self._hp = hp
        self.max_hp = max_hp

        # toughness is a way of determing a fair amount of damage from collisions
        # higher toughness means less damage taken from falls, bumps, etc

        # more massive actors should have a higher toughness.
        # currently, there is no conversion of mass done in the collisions 
        self.toughness = 2000

        self.is_living = True

    def destroy(self):
        self._hp = 0
        self.is_living = False

    @property
    def hp(self):
        return self._hp

    @hp.setter
    def hp(self, value):
        if value <= 0:
            self.destroy()
        else:
            self._hp = value

    def get_hp(self):
        """
        While I don't like using methods to access instance attributes, this
        is needed for gui elements and what not to get up-to-date data.

        returns a float 0-1
        """
        return self._hp / self.max_hp
         
class Ragdoll(Actor, SpriteBundle):
    def __init__(self, sprites, joints):
        SpriteBundle.__init__(self, sprites, joints)
        Actor.__init__(self)

        # HACK
        self.__Sprite__g = []
        self._Sprite__g = []

