"""
Copyright 2011,  Leif Theden & Michael Bachmann


This file is part of Nein.

Nein is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Nein is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Nein.  If not, see <http://www.gnu.org/licenses/>.
"""

"""
Some functions to quickly build PhysicsSprites
"""

import pygame
import Box2D as b2d
from sprite import PhysicsSprite
from actor import Ragdoll

# misc globals
# used to managing collisions
global_colgroup = 0
global_catbits  = 0x0000

def make_cat_ragdoll(world, origin):
    """
    Build our cat ragdoll =)
    """

    from random import randint

    global global_colgroup
    global global_catbits

    old_colgroup = global_colgroup
    old_catbits = global_catbits
   
    global_colgroup = -1

    origin_x, origin_y = origin

    # world coordinates, not pixels
    x, y = origin
    body_seg_x = 6
    body_seg_y = 6
    body_segments = 4

    tail_seg_x = 5
    tail_seg_y = 3
    tail_segments = 4

    head_width = 8
    head_height = 8

    leg_width = 5
    leg_length = 10

    # how bouncy the cat is.  between 0 and 1
    b = .70

    # how dense that cat is.  more or less the mass.
    # the actual value gets distributed to the parts unevenly
    # the mass of the cat total does not equal this value
    d = 50

    # dictionary to hold our joints
    # not sure how to organize it yet.  currently it is (sprite0, sprite1) - joint 
    # you can access the bodies by sprite.body, if needed
    j = {}

    # build the sprites for the tail
    tail = []
    for i in xrange(tail_segments):
        image = pygame.surface.Surface((tail_seg_x, tail_seg_y))
        image.fill((randint(240,255), randint(190, 230), randint(100, 130)))
        tail.append(make_sprite(world, image, pos=(x, y),\
                                density=d/2/tail_segments, r=b))
        x += tail_seg_x - 1

    # join them together with rev. joints
    rjd = b2d.b2RevoluteJointDef()
    rjd.enableLimit   = True
    rjd.lowerAngle    = -0.2 * b2d.b2_pi
    rjd.upperAngle    = 0.2 * b2d.b2_pi

    for s0, s1 in zip(tail[0::1], tail[1::1]):
        rjd.Initialize(s0.body, s1.body, s1.body.position)
        j[(s0, s1)] = world.world.CreateJoint(rjd)

    x -= 2
    y -= abs(body_seg_y - tail_seg_y)

    # build the sprites for the body
    body = []
    for i in xrange(body_segments):
        image = pygame.surface.Surface((body_seg_x, body_seg_y))
        image.fill((randint(220,255), randint(170, 230), randint(70, 130)))
        body.append(make_sprite(world, image, pos=(x, y), \
                                density=d/body_segments, r=b))
        x += body_seg_x - 4

    # join them together with rev. joints
    rjd.lowerAngle    = -0.1 * b2d.b2_pi
    rjd.upperAngle    = 0.0625 * b2d.b2_pi

    for s0, s1 in zip(body[0::1], body[1::1]):
        rjd.Initialize(s0.body, s1.body, s1.body.position)
        j[(s0, s1)] = world.world.CreateJoint(rjd)

    # join the tail to the body
    rjd.lowerAngle    = -0.2 * b2d.b2_pi
    rjd.upperAngle    = 0.2 * b2d.b2_pi
    rjd.Initialize(tail[-1].body, body[0].body, tail[-1].body.position)
    j[(tail[-1], body[0])] = world.world.CreateJoint(rjd)

    # the head
    y = origin_y - 4

    image = pygame.surface.Surface((head_width, head_height))
    image.fill((randint(210,220), randint(100, 150), randint(50, 100)))

    head = make_sprite(world, image, worldpos=body[-1].body.GetWorldCenter(), \
            density=d/10, r=b/3)

    rjd.Initialize(head.body, body[-1].body, body[-1].body.position)
    j[(head, body[-1])] = world.world.CreateJoint(rjd)

    # legs!
    global_colgroup = 2
    y -= leg_length - 2 
    limbs = []

    # just make the right side legs for now, since we dont have layered sprites

    y_offset = world.toWorld((x, leg_length))[1]

    # hind legs
    image = pygame.surface.Surface((leg_width, leg_length))
    image.fill((randint(200,220), randint(100, 150), randint(50, 100)))
    leg0 = make_sprite(world, image, \
        worldpos=(body[0].body.position[0], body[0].body.position[1] - y_offset), \
        density=d, r=b)
    limbs.append(leg0)

    rjd.lowerAngle    = -0.4 * b2d.b2_pi
    rjd.upperAngle    = 0.4 * b2d.b2_pi

    rjd.Initialize(leg0.body, body[0].body, body[0].body.position)
    j[(leg0, body[0])] = world.world.CreateJoint(rjd)

    # front legs
    image = pygame.surface.Surface((leg_width, leg_length))
    image.fill((randint(200,220), randint(100, 150), randint(50, 100)))
    leg1 = make_sprite(world, image, \
        worldpos=(body[-1].body.position[0], body[-1].body.position[1] - y_offset), \
        density=d, r=b)
    limbs.append(leg1)

    rjd.Initialize(leg1.body, body[-1].body, body[-1].body.position)
    j[(leg1, body[-1])] = world.world.CreateJoint(rjd)

    l = []
    l.extend(body)
    l.extend(tail)
    l.extend(limbs)
    l.append(head)

    global_colgroup = old_colgroup
    global_catbits = old_catbits

    ragdoll = Ragdoll(l, j)

    ragdoll.toughness = 10000

    return ragdoll


def build_shape_def(world, image=None, poly=None, density=1, friction=0.5, r=0):
    shapeDef = b2d.b2PolygonDef()
    shapeDef.filter.groupIndex = global_colgroup
    shapeDef.filter.catagoryBits = global_catbits
    shapeDef.density = density
    shapeDef.friction = friction
    shapeDef.restitution = r

    if image != None:
        r = image.get_rect()
        shapeDef.setVertices(world.coordsToMeters(r.topleft, r.topright, r.bottomright, r.bottomleft))
    elif poly != None:
        shapeDef.setVertices(poly)
    else:
        raise ValueError, "Cannot create shape def without data"

    return shapeDef


def build_body_def(world, pos=None, worldpos=None):
    bodyDef = b2d.b2BodyDef()

    if worldpos != None:
        bodyDef.position = worldpos
    elif pos != None:
        bodyDef.position = world.toWorld(pos)
    else:
        bodyDef.position = (20,20)

    return bodyDef

def build_body(world, image, poly=None, pos=None, worldpos=None, density=1, friction=0.5, r=0):
    """
    make a composite class that is a sprite, also a physics body and shape
    """

    bodyDef = build_body_def(world, pos, worldpos)
    body = world.world.CreateBody(bodyDef)
    shapeDef = build_shape_def(world, image, poly, density, friction, r)
    body.CreateShape(shapeDef)
    body.SetMassFromShapes()

    return body

def make_sprite(world, image, poly=None, pos=None, worldpos=None, density=1, friction=0.5, r=0):
    return PhysicsSprite(image, build_body(world, image, poly, pos, worldpos, density, friction, r))


def static_sprite(world, image, poly=None, pos=None, worldpos=None):
    """
    helper func makes a non-moving sprite on the level
    """
    return make_sprite(world, image, poly, pos, worldpos, density=0)

