"""
Copyright 2011,  Leif Theden & Michael Bachmann


This file is part of Nein.

Nein is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Nein is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Nein.  If not, see <http://www.gnu.org/licenses/>.
"""

from pygame.locals import *
from pygame.rect import Rect
from pygame.transform import flip
import pygame
from engine import EventHandler

# modified from
# http://www.pygame.org/pcr/gfxcursor/index.php
class GfxCursor(object):
    """
    Replaces the normal pygame cursor with any bitmap cursor
    """

    def __init__(self,cursor,hotspot=(0,0)):
        """
        surface = Global surface to draw on
        cursor  = surface of cursor (needs to be specified when enabled!)
        hotspot = the hotspot for your cursor
        """
        self.enabled = 0
        self.cursor  = None
        self.shadow  = None
        self.hotspot = hotspot
        self.bg      = None
        self.offset  = 0,0
        self.old_pos = 0,0
        self.direction = 0       
        self.do_flip = False
 
        if cursor:
            self.setCursor(cursor,hotspot)

    def enable(self):
        """
        Enable the GfxCursor (disable normal pygame cursor)
        """
        if not self.cursor or self.enabled: return
        pygame.mouse.set_visible(0)
        self.enabled = 1

    def disable(self):
        """
        Disable the GfxCursor (enable normal pygame cursor)
        """
        if self.enabled:
            pygame.mouse.set_visible(1)
            self.enabled = 0

    def make_shadow(self):
        # generate an image for use as a shadow
        # this is a kludge
        self.shadow = self.cursor.copy()
        colorkey = self.cursor.get_at((0,0))
        self.shadow.set_colorkey(colorkey)
        for x in xrange(self.cursor.get_rect().width):
            for y in xrange(self.cursor.get_rect().height):
                if self.shadow.get_at((x, y)) != colorkey:
                    self.shadow.set_at((x, y), (0,0,0))
        self.shadow.convert()
        self.shadow.set_alpha(60)

    def setCursor(self,cursor,hotspot=(0,0)):
        """
        Set a new cursor surface
        """
        self.cursor = cursor
        self.offset = hotspot
        self.cursor.set_alpha(200)
        self.make_shadow()
        if self.direction == 1:
            self.do_flip = True

    def setHotspot(self,pos):
        """
        Set a new hotspot for the cursor
        """
        self.hide()
        self.offset = pos

    def draw(self, surface):
        if self.enabled:
            if self.do_flip:
                self.cursor = flip(self.cursor, True, False)
                if self.shadow != None:
                    self.shadow = flip(self.shadow, True, False)
                self.do_flip = False

            pos = [self.old_pos[0]-self.offset[0], self.old_pos[1]-self.offset[1]]

            r = surface.get_rect()
            if pos[0] > r.right:
                pos[0] = r.right

            if pos[1] > r.bottom:
                pos[1] = r.bottom

            if self.shadow != None:
                surface.blit(self.shadow,(pos[0]+2, pos[1]+2))
            surface.blit(self.cursor,pos)


    def update(self, pos):
        if self.direction == 0:
            if self.old_pos[0] > pos[0]:
                if self.do_flip == False:
                    self.direction = 1
                    self.do_flip = True

        else:
            if self.old_pos[0] < pos[0]:
                if self.do_flip == False:
                    self.direction = 0
                    self.do_flip = True

        self.old_pos = pos

