"""
    Program: NeinLivesTitle
    Author: Mike Bachmann Leif Theden
    Date: 4-5-11
    Purpose:
"""

from resources.world import Level1
from pygame.locals import *
from pygame.sprite import Sprite, OrderedUpdates
from pygame.time import get_ticks
import pygame, random
from math import ceil
from lib.engine import GameState
import lib.gfx as gfx
import lib.sfx as sfx

miceAnim = ["SMouse1.gif", "SMouse2.gif"]

# BUG: skipping the intro messes up the credits, not time to fix it!


class StaticSprite(Sprite):
    def __init__(self, filename, pos, sound=None):
        Sprite.__init__(self)
        self.image = gfx.load(filename).convert()
        self.rect = self.image.get_rect()
        self.rect.topleft = pos
        if sound != None:
            self.sound = sfx.load(sound)
        else:
            self.sound = None

#Was going to add a zooming logo, thus the separate class
#Decided to just keep it classy (pun intended) instead
class zoomTitle(StaticSprite):
    def __init__(self, filename, pos, sound=None):
        StaticSprite.__init__(self, filename, pos, sound)
        self.image.set_colorkey((0,0,0))
        self.rect.center = (pos)

class CoolList(object):
    """
    Well... its pretty cooollll~~~

    (tho not prefect...)

    it would require numpy on the target platform to remove the
    bug where surfaces overwrite eachother's alpha values.

    timing is such that movement is smooth per pixel (complicated!!!)
    """

    def __init__(self, banner, pos, fg, bg, font_size, font_name, move_time=4500.0):
        font = pygame.font.Font(font_name, font_size)
        self.images = [ (i, i.get_rect()) for i in [ font.render(i, True, fg, bg) for i in banner ] ]
        self.bg = bg
        self.x, self.y = pos
        self.orig_speed = 30
        self.speed = 30         # this is set high to give the class some time to guess the fps
        self.delay = 0
        self.move_time = float(move_time)
        self.fade_time = move_time + float(move_time) / 5
        self.start = None
        self.finished = False
        self.offset = self.y
        self.index = 0
        self.calc_speed = False
        self.clock = pygame.time.Clock()

    def skip(self, surface):
        """
        basically, render all the image and plot them quickly, skipping the fade
        """

        [ i.set_colorkey(self.bg) for i in self.images ]

    def draw(self, surface):
        if self.finished:
            return

        now = get_ticks()

        if self.start == None:
             self.start = now

        self.delay += 1
        self.clock.tick()

        if self.index == len(self.images):
            return

        s0, r0 = self.images[self.index]

        # advance the image one pixel
        if self.delay >= self.speed:
            self.delay = 0
            self.offset += 1

            if self.calc_speed == False:
                fps = self.clock.get_fps()
                if fps > 0:
                    self.speed = int((r0.height * 6) * (self.move_time / 1000.0) / fps)
                    self.orig_speed = self.speed
                    self.calc_speed = True
                    return

        # this thing has finished moving
        if (now - self.start) >= self.move_time:
            if self.index < len(self.images):
                s0.set_colorkey((0,0,0))
                self.images[self.index] = (s0, r0) 
                self.offset += r0.height / 2
                self.start = now
                self.speed = self.orig_speed
                self.index += 1
            else:
                self.index -= 1
                union = []
                s0.set_colorkey(self.bg)
                surface.fill(self.bg, r0.unionall([ i[1] for i in union ]))
                surface.blit(s0, r0)
                [ surface.blit(i[0], i[1]) for i in self.images[:self.index] ]
                self.finished = True
            return

        # draw image with new alpha value
        alpha = int((now - self.start)/ self.fade_time * 255)
        if alpha >= 255:
            alpha = 255
        elif alpha > 200:
            self.speed += self.orig_speed / 4

        if self.index == 0: 
            s0.set_alpha(alpha)
            surface.fill((0,0,0), r0)
            r0.topleft = self.x, self.offset
            #r0.topleft = self.x, self.offset + int((now - self.start) / self.fade_time * (r0.height * 1.2))
            surface.blit(s0, r0)

        elif self.index < len(self.images):
            union = []
            r0.topleft = self.x, self.offset
            s0.set_alpha(alpha)
            surface.fill(self.bg, r0.unionall([ i[1] for i in union ]))
            surface.blit(s0, r0)
            [ surface.blit(i[0], i[1]) for i in self.images[:self.index] ]

class MouseSprite(Sprite):
    left_anim = [ gfx.load(f).convert() for f in miceAnim ]
    right_anim = [ pygame.transform.flip(s, True, False) for s in left_anim ]

    sound = sfx.load("squeak.wav")

    def __init__(self, pos, left):
        Sprite.__init__(self)
        # new sprites walk slower, for dramatic effect
        self.new = True
        self.left = left
        self.pos = pos
        self.frame = 0
        self.pause = 0
        self.reset()

    def update(self):
        if self.left:
            self.rect.centerx -= self.dx
        else:
            self.rect.centerx += self.dx

        self.pause +=1

        if self.pause >= self.delay:
            self.frame +=1
            self.pause = 0

            # this assumes that the left and right animations are same length
            if self.frame >= len(self.left_anim):
                self.frame = 0

            if self.left:
                self.image = self.left_anim[self.frame]
            else:
                self.image = self.right_anim[self.frame]

        if (self.rect.right < 0) or (self.rect.left > gfx.screen_dim[0]):
            self.reset()
        
    def reset(self):
        #if self.new:
        #    self.dx = random.randrange(1, 3)
        #    self.new = False
        #else:
        #    self.dx = random.randrange(1, 5)
        self.dx = 3 
        self.delay = 9 - self.dx

        if self.left:
            self.image = self.left_anim[0]
            self.rect = self.image.get_rect()
            self.rect.topleft = gfx.screen_dim[0], self.pos[1]
        else:
            self.image = self.right_anim[0]
            self.rect = self.image.get_rect()
            self.rect.topright = 0, self.pos[1]

class TitleScreen(GameState): 
    def __init__(self, parent):
        GameState.__init__(self, parent)

        # load some sprites
        self.naziCat = StaticSprite("grey_cat.gif", (300,50), "meow.wav")
        self.logo    = zoomTitle("nein.png", (500, 275))
        self.staticSprites = OrderedUpdates(self.naziCat, self.logo)
        self.message = StaticSprite("message.png", (0,0))
        self.message.rect.left = (gfx.screen_dim[0] / 2) - (self.message.rect.width / 2)
        self.message.rect.top = self.logo.rect.bottom + 20

        # HACK
        self.message.image = gfx.load_image_with_alpha("message.png")

        # only this portion of the screen gets cleared for mice
        self.redrawArea = (0, 425), gfx.screen_dim

        # bkg
        self.background = gfx.load("nazikitten.jpg").convert()

        # determine which element to play based on time
        self.steps = [0.0,400.0,3200.0,4000.0,6000.0,10000.0,10000.0,3000.0]

    def activate(self):
        self.intensity = 0
        sfx.load_music("EPBolero-lofi.ogg")

        mice = []
        for i in xrange(1,4):
            x = MouseSprite((1000,425), True)
            x.dx = i
            mice.append(x)

        for i in xrange(1,4):
            x = MouseSprite((1000,425), False)
            x.dx = i
            mice.append(x)

        self.sprites = pygame.sprite.Group(mice)

        # make a nice list of clickable sprites
        self.clickSprites = [self.naziCat]
        self.clickSprites.extend(self.sprites)

        # bet you wish you had one of these?
        self.credits = CoolList(["Leif Theden", "     &", "   Michael Bachmann's"],\
                    (40, 10), (100,100,100), (0,0,0), 32, "resources/crystaldeco.ttf")

        self.skipped_intro = False
        self.intro_finished = False
        self.step = 0
        self.fbdelay = 0
        self.fbspeed = 300
        self.t_oldx = 0
        self.fadeout = False

    def handle_event(self, event):
        if event.type == MOUSEBUTTONDOWN:
            for sprite in self.clickSprites:
                if sprite.rect.collidepoint(pygame.mouse.get_pos()):
                    sprite.sound.set_volume(0.3)
                    sprite.sound.play()

        if event.type == KEYDOWN:
            if event.key == K_SPACE:
                if self.intro_finished:
                    pygame.mixer.music.fadeout(2000)
                    self.fadeout = True
                    self.start = get_ticks()

            if not self.skipped_intro:
                # this is needed to make sure the message fades in properly
                self.skipped_intro = True
                self.start = get_ticks()
                self.naziCat.image.set_alpha(200)
                self.logo.image.set_alpha(255)
                self.step = 7
                self.fbspeed = 0

                self.credits.fade_time = 1
                self.credits.move_time = 100
                self.credits.speed = 1
                self.credits.orginal_speed = 1


    def draw(self, surface):
        now = get_ticks()

        # a transition into the next screen
        if self.fadeout:
            x = int((now-self.start) / 1500.0 * gfx.screen_dim[0])
            surface.fill((0,0,0), (0,0,x,gfx.screen_dim[1]))
            self.t_oldx = x
            if x >= gfx.screen_dim[0]:
                self._driver.done()

        # init the stepper thing
        if self.step == 0:
            self.start = now

        try:
            if (now - self.start) >= self.steps[self.step]:
                self.step += 1
                self.start = now
        except IndexError:
            pass

        # music
        if self.step == 1:
            sfx.play_music(-1) 
        
        # logo
        if self.step == 2:
            alpha = ceil((now-self.start) / self.steps[2] * 255)
            if alpha > 255: alpha = 255
            self.logo.image.set_alpha(alpha)
            surface.fill((0,0,0), self.logo.rect)
            surface.blit(self.logo.image, self.logo.rect)

        # cat
        elif self.step == 3:
            alpha = ceil((now-self.start) / self.steps[3] * 255)
            if alpha > 255: alpha = 255
            self.naziCat.image.set_alpha(alpha)
            surface.fill((0,0,0), self.logo.rect.union(self.naziCat.rect))
            surface.blit(self.naziCat.image, self.naziCat.rect)
            surface.blit(self.logo.image, self.logo.rect)

            # these will be blitted again in the next fade, just set their alpha here
            self.naziCat.image.set_alpha(200)
            self.logo.image.set_alpha(255)

        elif self.step == 7:
            if not self.intro_finished:
                surface.blit(self.message.image, self.message.rect)
                self.intro_finished = True

        # fade the background
        if self.step >= 6:
            if now - self.fbdelay >= self.fbspeed and (self.intensity < 25):
                self.intensity += 1
                self.fbdelay = now
                self.background.set_alpha(3)
                surface.blit(self.naziCat.image, self.naziCat.rect)
                surface.blit(self.logo.image, self.logo.rect)
                surface.blit(self.background, (0,0))

        # add credits
        if self.step >= 5:
            self.credits.draw(surface)

        # draw mice sprites.  
        if 3 < self.step < 6:
            self.sprites.update()
            surface.fill((0,0,0), self.redrawArea)
            self.sprites.draw(surface)

