"""
    Program: NeinLivesTitle
    Author: Mike Bachmann Leif Theden
    Date: 4-5-11
    Purpose:
"""

from pygame.transform import rotozoom
from pygame.locals import *
from pygame.sprite import Sprite, OrderedUpdates
from pygame.time import get_ticks
import pygame, random
from math import ceil
from lib.engine import GameState
import lib.gfx as gfx
import lib.sfx as sfx

miceAnim = ["DMouse1.gif", "DMouse2.gif"]

class StaticSprite(Sprite):
    def __init__(self, filename, pos, sound=None):
        Sprite.__init__(self)
        self.image = gfx.load(filename).convert()
        self.rect = self.image.get_rect()
        self.rect.topleft = pos
        if sound != None:
            self.sound = sfx.load(sound)
        else:
            self.sound = None

#Was going to add a zooming logo, thus the separate class
#Decided to just keep it classy (pun intended) instead
class zoomTitle(StaticSprite):
    def __init__(self, filename, pos, sound=None):
        StaticSprite.__init__(self, filename, pos, sound)
        self.image.set_colorkey((0,0,0))
        self.rect.center = (pos)

class CoolList(object):
    """
    Well... its pretty cooollll~~~

    (tho not prefect...)

    it would require numpy on the target platform to remove the
    bug where surfaces overwrite eachother's alpha values.

    timing is such that movement is smooth per pixel (complicated!!!)
    """

    def __init__(self, banner, pos, fg, bg, font_size, font_name, move_time=4500.0):
        font = pygame.font.Font(font_name, font_size)
        self.images = [ (i, i.get_rect()) for i in [ font.render(i, True, fg, bg) for i in banner ] ]
        self.bg = bg
        self.x, self.y = pos
        self.orig_speed = 30
        self.speed = 30         # this is set high to give the class some time to guess the fps
        self.delay = 0
        self.move_time = float(move_time)
        self.fade_time = move_time + float(move_time) / 5
        self.start = None
        self.finished = False
        self.offset = self.y
        self.index = 0
        self.calc_speed = False
        self.clock = pygame.time.Clock()

    def skip(self, surface):
        """
        basically, render all the image and plot them quickly, skipping the fade
        """

        [ i.set_colorkey(self.bg) for i in self.images ]

    def draw(self, surface):
        if self.finished:
            return

        now = get_ticks()

        if self.start == None:
             self.start = now

        self.delay += 1
        self.clock.tick()

        if self.index == len(self.images):
            return

        s0, r0 = self.images[self.index]

        # advance the image one pixel
        if self.delay >= self.speed:
            self.delay = 0
            self.offset += 1

            if self.calc_speed == False:
                fps = self.clock.get_fps()
                if fps > 0:
                    self.speed = int((r0.height * 6) * (self.move_time / 1000.0) / fps)
                    self.orig_speed = self.speed
                    self.calc_speed = True
                    return

        # this thing has finished moving
        if (now - self.start) >= self.move_time:
            if self.index < len(self.images):
                s0.set_colorkey((0,0,0))
                self.images[self.index] = (s0, r0) 
                self.offset += r0.height / 2
                self.start = now
                self.speed = self.orig_speed
                self.index += 1
            else:
                self.index -= 1
                union = []
                s0.set_colorkey(self.bg)
                surface.fill(self.bg, r0.unionall([ i[1] for i in union ]))
                surface.blit(s0, r0)
                [ surface.blit(i[0], i[1]) for i in self.images[:self.index] ]
                self.finished = True
            return

        # draw image with new alpha value
        alpha = int((now - self.start)/ self.fade_time * 255)
        if alpha >= 255:
            alpha = 255
        elif alpha > 200:
            self.speed += self.orig_speed / 4

        if self.index == 0: 
            s0.set_alpha(alpha)
            surface.fill((0,0,0), r0)
            r0.topleft = self.x, self.offset
            #r0.topleft = self.x, self.offset + int((now - self.start) / self.fade_time * (r0.height * 1.2))
            surface.blit(s0, r0)

        elif self.index < len(self.images):
            union = []
            r0.topleft = self.x, self.offset
            s0.set_alpha(alpha)
            surface.fill(self.bg, r0.unionall([ i[1] for i in union ]))
            surface.blit(s0, r0)
            [ surface.blit(i[0], i[1]) for i in self.images[:self.index] ]

class MouseSprite(Sprite):
    left_anim = [ gfx.load(f).convert() for f in miceAnim ]
    right_anim = [ pygame.transform.flip(s, True, False) for s in left_anim ]

    sound = sfx.load("squeak.wav")

    def __init__(self, pos, left):
        Sprite.__init__(self)
        self.left = left
        self.pos = pos
        self.frame = 0
        self.pause = 0
        self.delay = 10

        if self.left:
            self.image = self.left_anim[0]
        else:
            self.image = self.right_anim[0]

        self.rect = pygame.rect.Rect(pos, self.image.get_rect().size)

    def update(self):
        self.pause +=1

        if self.pause >= self.delay:
            self.frame +=1
            self.pause = 0

            # this assumes that the left and right animations are same length
            if self.frame >= len(self.left_anim):
                self.frame = 0

            if self.left:
                self.image = self.left_anim[self.frame]
            else:
                self.image = self.right_anim[self.frame]

            self.rect.size = self.image.get_rect().size

class HighScoreScreen(GameState): 
    def __init__(self, parent):
        GameState.__init__(self, parent)
        
    def activate(self):
        sfx.load_music("EPScoreCredits-lofi.ogg")
        sfx.play_music()

        self.t_oldx = 0
        self.fadeout = False
        self.drawnGfx = False

        self.walkMouse = MouseSprite((90, 270), True)
        self.backButton = StaticSprite("back.gif", (60, 320))

        # mash together a logo
        naziCat = StaticSprite("grey_cat.gif", (0,0))
        self.logo    = zoomTitle("nein.png", (200, 225), "meow.wav")
        s = pygame.surface.Surface(self.logo.rect.union(naziCat.rect).size)
        s.blit(naziCat.image, (0,0))
        s.blit(self.logo.image, self.logo.rect.topleft)
        self.logo.image = rotozoom(s, 0, 0.7)
        self.logo.rect = s.get_rect()

        # add the general
        self.naziCat = StaticSprite("standcat.png", (500,0), "drown.wav")
        self.naziCat.image = rotozoom(self.naziCat.image, 0, 0.25)
        self.naziCat.rect.size = self.naziCat.image.get_rect().size
        self.naziCat.rect.topleft = (gfx.screen_dim[0] - self.naziCat.rect.width+80,
            gfx.screen_dim[1] - self.naziCat.rect.height)

        head = StaticSprite("HighScores.gif", (360, 30))

        self.clickSprites = [self.logo, self.naziCat, self.walkMouse, head]

        self.staticSprites = OrderedUpdates(self.logo, self.naziCat, self.backButton, head)

        # get a list of high scores
        scores = []
        file = open("scores.txt")
        for x in range(1,11):
            line = file.readline().strip()
            if line:
                scores.append(str(x) + ".  " + line)
            else:
                break
 
        # yeah man, its cool
        self.scores = CoolList(
            scores,
            (gfx.screen_dim[0] / 2 - 100, 100),
            (255,255,255),
            (0,0,0),
            40,
            "resources/crystaldeco.ttf",
            1000.0)

    def handle_event(self, event):
        if event.type == MOUSEBUTTONDOWN:
            for sprite in self.clickSprites:
                if sprite.rect.collidepoint(pygame.mouse.get_pos()):
                    sprite.sound.set_volume(0.3)
                    sprite.sound.play()

            if self.backButton.rect.collidepoint(pygame.mouse.get_pos()):
                self.start = get_ticks()
                pygame.mixer.music.fadeout(1500)
                self.fadeout = True

    def draw(self, surface):
        now = get_ticks()


        # a transition into the next screen
        if self.fadeout:
            x = int((now-self.start) / 1500.0 * gfx.screen_dim[0])
            surface.fill((0,0,0), (0,0,x,gfx.screen_dim[1]))
            self.t_oldx = x
            if x >= gfx.screen_dim[0]:
                self._driver.done()
            return

        self.scores.draw(surface)
        self.staticSprites.draw(surface)
        surface.fill((0,0,0), self.walkMouse.rect)
        self.walkMouse.update()
        surface.blit(self.walkMouse.image, self.walkMouse.rect)
